##################### generated by xml-casa (v2) from cvel.xml ######################
##################### 0547fb425e17ddf38c97521fa4ec472a ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_cvel import cvel as _cvel_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _cvel:
    """
    cvel ---- regrid an MS to a new spectral window / channel structure or frame

    
    The intent of cvel is to transform channel labels and the
    visibilities to a spectral reference frame which is appropriate
    for the science analysis, e.g. from TOPO to LSRK to correct for
    Doppler shifts throughout the time of the observation. Naturally,
    this will change the shape of the spectral feature to some extent.
    According to the Nyquist theorem you should oversample a spectrum
    with twice the numbers of channels to retain the shape. Based on
    some tests, however, we recommend to observe with at least
    3-4 times the number of channels for each significant spectral
    feature (like 3-4 times the linewidth). This will minimize
    regridding artifacts in cvel.
    
    If cvel has already established the grid that is desired for the
    imaging, clean should be run with exactly the same frequency/velocity
    parameters as used in cvel in order to avoid additional regridding in
    clean.
    
    Hanning smoothing is optionally offered in cvel, but tests have
    shown that already the regridding process itself, if it involved
    a transformation from TOPO to a non-terrestrial reference frame,
    implies some smoothing (due to channel interpolation) such that
    Hanning smoothing may not be necessary.

    --------- parameter descriptions ---------------------------------------------

    vis           Name of input measurement set
    outputvis     Name of output measurement set
    passall       Pass through (write to output MS) non-selected data with no change
    field         Select field using field id(s) or field name(s)
    spw           Select spectral window/channels
    selectdata    Other data selection parameters
    antenna       Select data based on antenna/baseline
    timerange     Range of time to select from data
    scan          scan number range
    array         (sub)array indices
    mode          Regridding mode
    nchan         Number of channels in output spw (-1=all). Used for regridding, together with \'start\' and \'width\'.
    start         Start of the output visibilities. Used for regridding, together with \'width\' and \'nchan\'. It can be in different units, depending on the regridding mode: first input channel (mode=\'channel\'), first velocity (mode=\'velocity\'), or first frequency (mode=\'frequency\'). Example values: \'5\', \'0.0km/s\', \'1.4GHz\', for channel, velocity, and frequency modes, respectively.
    width         Channel width of the output visibilities. Used for regridding, together with \'start\', and \'nchan\'. It can be in different units, depending on the regridding mode: number of input channels (mode=\'channel\'), velocity (mode=\'velocity\'), or frequency (mode=\'frequency\'. Example values: \'2\', \'1.0km/s\', \'1.0kHz\', for channel, velocity, and frequency modes, respectively.
    interpolation Spectral interpolation method
    phasecenter   Phase center direction to be used for the spectral coordinate transformation: direction measure or field index
    restfreq      rest frequency (see help)
    outframe      Output frame (not case-sensitive, \'\'=keep input frame)
    veltype       velocity definition
    hanning       If true, Hanning smooth data before regridding to remove Gibbs ringing.
    [1;42mRETURNS[1;m          void

    --------- examples -----------------------------------------------------------

    
    
    vis -- Name of input visibility file
    default: none; example: vis='ngc5921.ms'
    
    outputvis -- Name of output measurement set (required)
    default: none; example: vis='ngc5921-regridded.ms'
    
    passall --  if False, data not meeting the selection is omitted/deleted
    or flagged (if in-row); if True, data not meeting the selection
    on field and spw is passed through without modification
    default: False; example:
    field='NGC5921'
    passall=False : only data from NGC5921 is included in output MS,
    no data from other fields (e.g. 1331+305) is included
    passall=True : data from NGC5921 is transformed by cvel, all other
    fields are passed through unchanged
    
    field -- Select fields in mosaic.  Use field id(s) or field name(s).
    ['go listobs' to obtain the list id's or names]
    default: ''= all fields
    If field string is a non-negative integer, it is assumed to
    be a field index otherwise, it is assumed to be a
    field name
    field='0~2'; field ids 0,1,2
    field='0,4,5~7'; field ids 0,4,5,6,7
    field='3C286,3C295'; field named 3C286 and 3C295
    field = '3,4C*'; field id 3, all names starting with 4C
    
    spw --Select spectral window/channels
    NOTE: This selects the data passed as the INPUT to mode
    default: ''=all spectral windows and channels
    spw='0~2,4'; spectral windows 0,1,2,4 (all channels)
    spw='0:5~61'; spw 0, channels 5 to 61
    spw='<2';   spectral windows less than 2 (i.e. 0,1)
    spw='0,10,3:3~45'; spw 0,10 all channels, spw 3,
    channels 3 to 45.
    spw='0~2:2~6'; spw 0,1,2 with channels 2 through 6 in each.
    spw='0:0~10;15~60'; spectral window 0 with channels
    0-10,15-60
    spw='0:0~10,1:20~30,2:1;2;3'; spw 0, channels 0-10,
    spw 1, channels 20-30, and spw 2, channels, 1,2 and 3
    
    selectdata -- Other data selection parameters
    default: True
    
    >>> selectdata=True expandable parameters
    
    antenna -- Select data based on antenna/baseline
    default: '' (all)
    If antenna string is a non-negative integer, it is
    assumed to be an antenna index, otherwise, it is
    considered an antenna name.
    antenna='5&6'; baseline between antenna index 5 and
    index 6.
    antenna='VA05&VA06'; baseline between VLA antenna 5
    and 6.
    antenna='5&6;7&8'; baselines 5-6 and 7-8
    antenna='5'; all baselines with antenna index 5
    antenna='05'; all baselines with antenna number 05
    (VLA old name)
    antenna='5,6,9'; all baselines with antennas 5,6,9
    index numbers
    
    timerange  -- Select data based on time range:
    default = '' (all); examples,
    timerange = 'YYYY/MM/DD/hh:mm:ss~YYYY/MM/DD/hh:mm:ss'
    Note: if YYYY/MM/DD is missing date defaults to first
    day in data set
    timerange='09:14:0~09:54:0' picks 40 min on first day
    timerange= '25:00:00~27:30:00' picks 1 hr to 3 hr
    30min on NEXT day
    timerange='09:44:00' pick data within one integration
    of time
    timerange='>10:24:00' data after this time
    
    scan -- Scan number range.
    default: '' (all)
    example: scan='1~5'
    Check 'go listobs' to insure the scan numbers are in
    order.
    
    array -- Select data by (sub)array indices
    default: '' (all); example:
    array='0~2'; arrays 0 to 2
    
    mode -- Frequency Specification:
    NOTE: See examples below:
    default: 'channel'
    mode = 'channel'; Use with nchan, start, width to specify
    output spw. Produces equidistant grid based on first
    selected channel. See examples below.
    mode = 'velocity', means channels are specified in
    velocity.
    mode = 'frequency', means channels are specified in
    frequency.
    mode = 'channel_b', alternative 'channel' mode.
    Does not force an equidistant grid. Faster.
    
    >>> mode expandable parameters
    Start, width are given in units of channels, frequency
    or velocity as indicated by mode
    nchan -- Number of channels in output spw
    default: -1 = all channels; example: nchan=3
    start -- Start or end input channel (zero-based) depending on the sign of the width parameter
    default=0; example: start=5
    width -- Output channel width in units of the input
    channel width (sign indicates whether the start parameter is lower(+) or upper(-) end of the range)
    default=1; example: width=4
    interpolation -- Interpolation method (linear, nearest, cubic, spline, fftshift)
    default = 'linear'
    examples:
    spw = '0,1'; mode = 'channel'
    will produce a single spw containing all channels in spw
    0 and 1
    spw='0:5~28^2'; mode = 'channel'
    will produce a single spw made with channels
    (5,7,9,...,25,27)
    spw = '0'; mode = 'channel': nchan=3; start=5; width=4
    will produce an spw with 3 output channels
    new channel 1 contains data from channels (5+6+7+8)
    new channel 2 contains data from channels (9+10+11+12)
    new channel 3 contains data from channels (13+14+15+16)
    spw = '0:0~63^3'; mode='channel'; nchan=21; start = 0;
    width = 1
    will produce an spw with 21 channels
    new channel 1 contains data from channel 0
    new channel 2 contains data from channel 2
    new channel 21 contains data from channel 61
    spw = '0:0~40^2'; mode = 'channel'; nchan = 3; start =
    5; width = 4
    will produce an spw with three output channels
    new channel 1 contains channels (5,7)
    new channel 2 contains channels (13,15)
    new channel 3 contains channels (21,23)
    
    phasecenter -- Direction measure  or fieldid. To be used in mosaics to indicate
    the center direction to be used in the spectral coordinate transformation.
    default: '' => first field selected ; example: phasecenter=6
    or phasecenter='J2000 19h30m00 -40d00m00'
    
    restfreq -- Specify rest frequency to use for output visibilities
    default='' Occasionally it is necessary to set this (for
    example some VLA spectral line data).  For example for
    NH_3 (1,1) put restfreq='23.694496GHz'
    
    outframe -- output reference frame (not case-sensitive)
    possible values: LSRK, LSRD, BARY, GALACTO, LGROUP, CMB, GEO, TOPO, or SOURCE
    (SOURCE is meant for solar system work and corresponds to GEO + radial velocity
    correction for ephemeris objects).
    default='' (keep original reference frame) ; example: outframe='BARY'
    
    veltype -- definition of velocity (in mode)
    default = 'radio'
    
    hanning -- if true, Hanning smooth frequency channel data to remove Gibbs ringing
    
    ==================================================================
    
    The intent of cvel is to transform channel labels and the
    visibilities to a spectral reference frame which is appropriate
    for the science analysis, e.g. from TOPO to LSRK to correct for
    Doppler shifts throughout the time of the observation. Naturally,
    this will change the shape of the spectral feature to some extent.
    According to the Nyquist theorem you should oversample a spectrum
    with twice the numbers of channels to retain the shape. Based on
    some tests, however, we recommend to observe with at least
    3-4 times the number of channels for each significant spectral
    feature (like 3-4 times the linewidth). This will minimize
    regridding artifacts in cvel.
    
    If cvel has already established the grid that is desired for the
    imaging, clean should be run with exactly the same frequency/velocity
    parameters as used in cvel in order to avoid additional regridding in
    clean.
    
    Hanning smoothing is optionally offered in cvel, but tests have
    shown that already the regridding process itself, if it involved
    a transformation from TOPO to a non-terrestrial reference frame,
    implies some smoothing (due to channel interpolation) such that
    Hanning smoothing may not be necessary.


    """

    _info_group_ = """manipulation"""
    _info_desc_ = """regrid an MS to a new spectral window / channel structure or frame"""

    def __call__( self, vis='', outputvis='', passall=False, field='', spw='', selectdata=True, antenna='', timerange='', scan='', array='', mode='channel', nchan=int(-1), start=int(0), width=int(1), interpolation='linear', phasecenter='', restfreq='', outframe='', veltype='radio', hanning=False ):
        schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'outputvis': {'type': 'cStr', 'coerce': _coerce.to_str}, 'passall': {'type': 'cBool'}, 'field': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}, {'type': 'cInt'}, {'type': 'cIntVec', 'coerce': [_coerce.to_list,_coerce.to_intvec]}]}, 'spw': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}, {'type': 'cInt'}, {'type': 'cIntVec', 'coerce': [_coerce.to_list,_coerce.to_intvec]}]}, 'selectdata': {'type': 'cBool'}, 'antenna': {'type': 'cStr', 'coerce': _coerce.to_str}, 'timerange': {'type': 'cStr', 'coerce': _coerce.to_str}, 'scan': {'type': 'cStr', 'coerce': _coerce.to_str}, 'array': {'type': 'cStr', 'coerce': _coerce.to_str}, 'mode': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'channel', 'velocity', 'frequency', 'channel_b' ]}, 'nchan': {'type': 'cInt'}, 'start': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'width': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'interpolation': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'nearest', 'linear', 'NEAREST', 'FFTSHIFT', 'spline', 'fftshift', 'LINEAR', 'SPLINE', 'CUBIC', 'cubic' ]}, 'phasecenter': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'restfreq': {'type': 'cStr', 'coerce': _coerce.to_str}, 'outframe': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'SOURCE', 'LSRK', 'CMB', 'geo', 'source', 'bary', 'GALACTO', 'GEO', 'TOPO', 'LSRD', 'lsrk', 'topo', 'LGROUP', 'BARY', 'lsrd', 'cmb', 'galacto', 'lgroup', '' ]}, 'veltype': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'optical', 'radio', 'OPTICAL', 'RADIO' ]}, 'hanning': {'type': 'cBool'}}
        doc = {'vis': vis, 'outputvis': outputvis, 'passall': passall, 'field': field, 'spw': spw, 'selectdata': selectdata, 'antenna': antenna, 'timerange': timerange, 'scan': scan, 'array': array, 'mode': mode, 'nchan': nchan, 'start': start, 'width': width, 'interpolation': interpolation, 'phasecenter': phasecenter, 'restfreq': restfreq, 'outframe': outframe, 'veltype': veltype, 'hanning': hanning}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'cvel', [ 'vis=' + repr(_pc.document['vis']), 'outputvis=' + repr(_pc.document['outputvis']), 'passall=' + repr(_pc.document['passall']), 'field=' + repr(_pc.document['field']), 'spw=' + repr(_pc.document['spw']), 'selectdata=' + repr(_pc.document['selectdata']), 'antenna=' + repr(_pc.document['antenna']), 'timerange=' + repr(_pc.document['timerange']), 'scan=' + repr(_pc.document['scan']), 'array=' + repr(_pc.document['array']), 'mode=' + repr(_pc.document['mode']), 'nchan=' + repr(_pc.document['nchan']), 'start=' + repr(_pc.document['start']), 'width=' + repr(_pc.document['width']), 'interpolation=' + repr(_pc.document['interpolation']), 'phasecenter=' + repr(_pc.document['phasecenter']), 'restfreq=' + repr(_pc.document['restfreq']), 'outframe=' + repr(_pc.document['outframe']), 'veltype=' + repr(_pc.document['veltype']), 'hanning=' + repr(_pc.document['hanning']) ] )
        task_result = None
        try:
            task_result = _cvel_t( _pc.document['vis'], _pc.document['outputvis'], _pc.document['passall'], _pc.document['field'], _pc.document['spw'], _pc.document['selectdata'], _pc.document['antenna'], _pc.document['timerange'], _pc.document['scan'], _pc.document['array'], _pc.document['mode'], _pc.document['nchan'], _pc.document['start'], _pc.document['width'], _pc.document['interpolation'], _pc.document['phasecenter'], _pc.document['restfreq'], _pc.document['outframe'], _pc.document['veltype'], _pc.document['hanning'] )
        except Exception as exc:
            _except_log('cvel', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'cvel', task_result )
        return task_result

cvel = _cvel( )

