##################### generated by xml-casa (v2) from imcontsub.xml #################
##################### 49e0cb68486224f2c3f83b64e80ca5b4 ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_imcontsub import imcontsub as _imcontsub_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _imcontsub:
    """
    imcontsub ---- Estimates and subtracts continuum emission from an image cube

    
    Estimates and subtracts continuum emission from an image cube
    
    For each direction pixel (x, y) column in imagename (or a subset
    selected by region and/or box), this estimates the continuum by
    fitting a polynomial to one or more subsets of the channels.  The
    continuum estimate is saved in contfile, and subtracted from imagename
    (or its subset) to make a spectral line estimate, which is saved in
    linefile.

    --------- parameter descriptions ---------------------------------------------

    imagename Input image cube.
              Default: none
              
                 Example: imagename='ngc5921_task.im'
    linefile  Name of continuum-subtracted output spectral line cube
              Default: none
              
                 Example: outline='ngc5921_line.im'
    contfile  Name of output continuum cube
              Default: none
              
                 Example: contfile='ngc5921_cont.im'
    fitorder  Polynomial order for the continuum estimation
              Default: 0
              
                 Example: fitorder=2
    region    Region selection. 
              Default: '' (use the full image)
    box       Rectangular region to select in direction plane.
              Default: '' (use the entire direction plane)
    chans     Channels to use. 
              Default: '' (use all channels)
    stokes    Stokes planes to use.
              Default: '' (use all Stokes planes)
    [1;42mRETURNS[1;m      void

    --------- examples -----------------------------------------------------------

    
    FOR MORE INFORMATION, SEE THE TASK PAGES OF IMCONTSUB IN CASA DOCS:
    https://casa.nrao.edu/casadocs/


    """

    _info_group_ = """analysis, imaging"""
    _info_desc_ = """Estimates and subtracts continuum emission from an image cube"""

    def __call__( self, imagename='', linefile='', contfile='', fitorder=int(0), region='', box='', chans='', stokes='' ):
        schema = {'imagename': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'linefile': {'type': 'cStr', 'coerce': _coerce.to_str}, 'contfile': {'type': 'cStr', 'coerce': _coerce.to_str}, 'fitorder': {'type': 'cInt', 'min': 0}, 'region': {'type': 'cStr', 'coerce': _coerce.to_str}, 'box': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cIntVec', 'coerce': [_coerce.to_list,_coerce.to_intvec]}, {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}]}, 'chans': {'type': 'cStr', 'coerce': _coerce.to_str}, 'stokes': {'type': 'cStr', 'coerce': _coerce.to_str}}
        doc = {'imagename': imagename, 'linefile': linefile, 'contfile': contfile, 'fitorder': fitorder, 'region': region, 'box': box, 'chans': chans, 'stokes': stokes}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'imcontsub', [ 'imagename=' + repr(_pc.document['imagename']), 'linefile=' + repr(_pc.document['linefile']), 'contfile=' + repr(_pc.document['contfile']), 'fitorder=' + repr(_pc.document['fitorder']), 'region=' + repr(_pc.document['region']), 'box=' + repr(_pc.document['box']), 'chans=' + repr(_pc.document['chans']), 'stokes=' + repr(_pc.document['stokes']) ] )
        task_result = None
        try:
            task_result = _imcontsub_t( _pc.document['imagename'], _pc.document['linefile'], _pc.document['contfile'], _pc.document['fitorder'], _pc.document['region'], _pc.document['box'], _pc.document['chans'], _pc.document['stokes'] )
        except Exception as exc:
            _except_log('imcontsub', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'imcontsub', task_result )
        return task_result

imcontsub = _imcontsub( )

