##################### generated by xml-casa (v2) from importvla.xml #################
##################### b803d2294e3bbe2af64178265f4004b5 ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_importvla import importvla as _importvla_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _importvla:
    """
    importvla ---- Import VLA archive file(s) to a measurement set

    
    Imports an arbitrary number of VLA archive-format data sets into a
    casa measurement set.  If more than one band is present, they will be
    put in the same measurement set but in a separate spectral window.
    The task will handle old style and new style VLA (after July 2007)
    archive data and apply the tsys to the data and to the weights.

    --------- parameter descriptions ---------------------------------------------

    archivefiles  Name of input VLA archive file(s)
                  Default: none.  Must be supplied
                  
                     Examples: 
                     archivefiles = 'AP314_A959519.xp1'
                     archivefiles=['AP314_A950519.xp1',
                     'AP314_A950519.xp2']
    vis           Name of output visibility file
                  Default: none.  Must be supplied
                  
                     Example: vis='NGC7538.ms'
                  
                  NOTE: Will not over-write existing ms of same
                  name. A backup flag-file version 'Original' will
                  be made in vis.flagversions.  See help
                  flagmanager.
    bandname      VLA frequency band name:
                  Default: '' (obtain all bands in the archive
                  file)
                  Options: '4'=48-96 MHz,'P'=298-345
                  MHz,'L'=1.15-1.75 GHz, 'C'=4.2-5.1
                  GHz,'X'=6.8-9.6 GHz,'U'=13.5-16.3 GHz,
                  'K'=20.8-25.8 GHz,'Q'=38-51 GHz
                  
                     Example: bandname='K'
    frequencytol  Tolerance in frequency shift in making spectral windows
                  Default: = 150000.0Hz'
                  
                     Example: frequencytol = 1500000.0 (units = Hz)
                  
                  For Doppler shifted data, less than 10000 Hz may
                  may produce too many unnecessary spectral
                  windows.
    project       Project name to import from archive files
                  Default: '' (all projects in file)
                  
                     Example: project='AL519'             
                     Project = 'al519' or AL519 will work. 
                  
                  WARNING: Do not include leading zeros; project =
                  'AL0519' will not work.
    starttime     Time after which data will be considered for importing
                  Default: '' (all)
                  
                  syntax: starttime = '2003/1/31/05:05:23'. Date
                  must be included!
    stoptime      Time before which data will be considered for
                  importing
                  Default: '' (all)
                  
                  syntax: starttime = '2003/1/31/08:05:23'. Date
                  must be included!
    applytsys     Apply data scaling and weight scaling by nominal
                  sensitivity (~Tsys)
                  Default: True (strongly recommended)
                  Options: True|False
    autocorr      Import autocorrelations to MS
                  Default: False (no autocorrelations)
                  Options: False|True
    antnamescheme 'old' or 'new' antenna names.
                  Default: 'new'
                  Options: new|old
                  
                  * 'new' gives antnenna names 'VA04' or 'EA13 for
                    VLA telescopse 04 and 13 (EVLA)
                  * 'old' gives names '04' or '13'
    keepblanks    Should sources with blank names be filled into the data
                  base?
                  Default: False (do not fill)
                  Options: False|True
                  
                  These scans are tipping scans (as of June 1,
                  2009) and should not be filled in the visibility
                  data set.
    evlabands     Use the EVLA's center frequency and bandwidths for
                  frequencies specified via wavelength or band.
                  Default: False
                  Options: False|True

    --------- examples -----------------------------------------------------------

    
    FOR MORE INFORMATION, SEE THE TASK PAGES OF IMPORTVLA IN CASA DOCS:
    https://casa.nrao.edu/casadocs/
    


    """

    _info_group_ = """import/export"""
    _info_desc_ = """Import VLA archive file(s) to a measurement set"""

    def __call__( self, archivefiles=[  ], vis='', bandname='', frequencytol='150000.0Hz', project='', starttime='', stoptime='', applytsys=True, autocorr=False, antnamescheme='new', keepblanks=False, evlabands=False ):
        schema = {'archivefiles': {'type': 'cReqPathVec', 'coerce': [_coerce.to_list,_coerce.expand_pathvec]}, 'vis': {'type': 'cStr', 'coerce': _coerce.to_str}, 'bandname': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ '4', 'X', 'S', 'P', 'K', 'Ka', 'Q', 'U', 'C', 'L', '' ]}, 'frequencytol': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'project': {'type': 'cStr', 'coerce': _coerce.to_str}, 'starttime': {'type': 'cStr', 'coerce': _coerce.to_str}, 'stoptime': {'type': 'cStr', 'coerce': _coerce.to_str}, 'applytsys': {'type': 'cBool'}, 'autocorr': {'type': 'cBool'}, 'antnamescheme': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'old', 'new' ]}, 'keepblanks': {'type': 'cBool'}, 'evlabands': {'type': 'cBool'}}
        doc = {'archivefiles': archivefiles, 'vis': vis, 'bandname': bandname, 'frequencytol': frequencytol, 'project': project, 'starttime': starttime, 'stoptime': stoptime, 'applytsys': applytsys, 'autocorr': autocorr, 'antnamescheme': antnamescheme, 'keepblanks': keepblanks, 'evlabands': evlabands}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'importvla', [ 'archivefiles=' + repr(_pc.document['archivefiles']), 'vis=' + repr(_pc.document['vis']), 'bandname=' + repr(_pc.document['bandname']), 'frequencytol=' + repr(_pc.document['frequencytol']), 'project=' + repr(_pc.document['project']), 'starttime=' + repr(_pc.document['starttime']), 'stoptime=' + repr(_pc.document['stoptime']), 'applytsys=' + repr(_pc.document['applytsys']), 'autocorr=' + repr(_pc.document['autocorr']), 'antnamescheme=' + repr(_pc.document['antnamescheme']), 'keepblanks=' + repr(_pc.document['keepblanks']), 'evlabands=' + repr(_pc.document['evlabands']) ] )
        task_result = None
        try:
            task_result = _importvla_t( _pc.document['archivefiles'], _pc.document['vis'], _pc.document['bandname'], _pc.document['frequencytol'], _pc.document['project'], _pc.document['starttime'], _pc.document['stoptime'], _pc.document['applytsys'], _pc.document['autocorr'], _pc.document['antnamescheme'], _pc.document['keepblanks'], _pc.document['evlabands'] )
        except Exception as exc:
            _except_log('importvla', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'importvla', task_result )
        return task_result

importvla = _importvla( )

