##################### generated by xml-casa (v2) from listvis.xml ###################
##################### 74d4b217d2565f7f95a58f1b6a2778be ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_listvis import listvis as _listvis_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _listvis:
    """
    listvis ---- List measurement set visibilities.

    
    
    This task lists measurement set visibility data under a number of
    input selection conditions.  The measurement set data columns that
    can be listed are: the raw data, float_data, corrected data, model data,
    and residual (corrected - model) data.
    
    The output table format is dynamic.  Field, Spectral Window, and
    Channel columns are not displayed if the column contents are uniform.
    For example, if "spw = '1'" is specified, the spw column will not be
    displayed.  When a column is not displayed, a message is sent to the
    logger and terminal indicating that the column values are uniform and
    listing the uniform value.
    
    Table column descriptions:
    
    COLUMN NAME       DESCRIPTION
    -----------       -----------
    Date/Time         Time stamp of data sample (YYMMDD/HH:MM:SS UT)
    Intrf             Interferometer baseline (antenna names)
    UVDist            uv-distance (units of wavelength)
    Fld               Field ID (if more than 1)
    SpW               Spectral Window ID (if more than 1)
    Chn               Channel number (if more than 1)
    (Correlated       Correlated polarizations (eg: RR, LL, XY)
    polarization)     Sub-columns are: Amp, Phs, Wt, F
    Amp               Visibility amplitude
    Phs               Visibility phase (deg)
    Wt                Weight of visibility measurement
    F                 Flag: 'F' = flagged datum; ' ' = unflagged
    UVW               UVW coordinates (meters)
    
    
    Input Parameters:
    vis         Name of input visibility file
    default: none; example: vis='ngc5921.ms'
    
    options     List options: default = 'ap'
    Not yet implemented for suboptions
    
    datacolumn  Visibility file data column:
    default = 'data':  options are
    data, float_data, corrected, model, residual (corrected-model)
    
    field       Select data based on field id(s) or name(s)
    default: ''==>all; example: field='1'
    field='0~2' field ids inclusive from 0 to 2
    field='3C*' all field names starting with 3C
    
    spw         Select spectral window, channel to list
    default: '0:0' --> spw=0, chan=0
    spw='2:34' spectral window 2, channel 34
    
    selectdata  Toggle the following 7 selection parameters.
    default: False; example: selectdata=True
    If false, the following parameters are reset
    to default values.
    
    antenna     Select calibration data based on antenna
    default: ''-->all; examples:
    antenna = '5,6'; antenna index 5 and 6 solutions
    antenna = '05,06'; antenna names '05' and '06 solutions
    
    timerange   Select time range to list
    default: ''-->all; examples:
    timerange='10:37:50.1'; list data for this sampling interval
    timerange='<10:37:25'; list data before 10:37:25
    
    correlation Select polarization correlations to list
    default: ''-->all; examples:
    correlation='RR LL'; list RR and LL correlations
    correlation='XX XY'; list XX and XY correlations
    
    scan        Select scans to list
    default: ''-->all; examples:
    scan='2'; list scan 2
    scan='>2'; list scan numbers greater than 2
    
    feed        (not yet implemented)
    
    array       (not yet implemented)
    
    observation Select by observation ID.
    
    uvrange     Select baseline lengths to list.
    default: ''--> all; examples:
    uvrange='<5klambda'; less than 5 kilo-wavelengths
    Caution: Input units default to meters.
    Listed units are always wavelengths.
    
    average     (not yet implemented)
    
    showflags   (not yet implemented)
    
    pagerows    rows per page of listing
    default: 50; 0 --> do not paginate
    
    listfile    write output to disk; will not overwrite
    default: '' --> write to screen
    listfile = 'solutions.txt'
    
    

    --------- parameter descriptions ---------------------------------------------

    vis         Name of input visibility file
    options     List options: ap only
    datacolumn  Column to list: data, float_data, corrected, model, residual
    field       Field names or index to be listed
    spw         Spectral window channels
    selectdata  Other data selection parameters
    antenna     Antenna/baselines
    timerange   Time range
    correlation Correlations
    scan        Scan numbers
    feed        Multi-feed numbers (Not yet implemented)
    array       Array numbers (Not yet implemented)
    observation Select by observation ID(s)
    uvrange     uv range
    average     Averaging mode
    showflags   Show flagged data (Not yet implemented)
    pagerows    Rows per page
    listfile    Output file
    [1;42mRETURNS[1;m        void

    --------- examples -----------------------------------------------------------

    
    
    This task lists measurement set visibility data under a number of
    input selection conditions.  The measurement set data columns that
    can be listed are: the raw data, float_data, corrected data, model data,
    and residual (corrected - model) data.
    
    The output table format is dynamic.  Field, Spectral Window, and
    Channel columns are not displayed if the column contents are uniform.
    For example, if "spw = '1'" is specified, the spw column will not be
    displayed.  When a column is not displayed, a message is sent to the
    logger and terminal indicating that the column values are uniform and
    listing the uniform value.
    
    Table column descriptions:
    
    COLUMN NAME       DESCRIPTION
    -----------       -----------
    Date/Time         Time stamp of data sample (YYMMDD/HH:MM:SS UT)
    Intrf             Interferometer baseline (antenna names)
    UVDist            uv-distance (units of wavelength)
    Fld               Field ID (if more than 1)
    SpW               Spectral Window ID (if more than 1)
    Chn               Channel number (if more than 1)
    (Correlated       Correlated polarizations (eg: RR, LL, XY)
    polarization)     Sub-columns are: Amp, Phs, Wt, F
    Amp               Visibility amplitude
    Phs               Visibility phase (deg)
    Wt                Weight of visibility measurement
    F                 Flag: 'F' = flagged datum; ' ' = unflagged
    UVW               UVW coordinates (meters)
    
    
    Input Parameters:
    vis         Name of input visibility file
    default: none; example: vis='ngc5921.ms'
    
    options     List options: default = 'ap'
    Not yet implemented for suboptions
    
    datacolumn  Visibility file data column:
    default = 'data':  options are
    data, float_data, corrected, model, residual (corrected-model)
    
    field       Select data based on field id(s) or name(s)
    default: ''==>all; example: field='1'
    field='0~2' field ids inclusive from 0 to 2
    field='3C*' all field names starting with 3C
    
    spw         Select spectral window, channel to list
    default: '0:0' --> spw=0, chan=0
    spw='2:34' spectral window 2, channel 34
    
    selectdata  Toggle the following 7 selection parameters.
    default: False; example: selectdata=True
    If false, the following parameters are reset
    to default values.
    
    antenna     Select calibration data based on antenna
    default: ''-->all; examples:
    antenna = '5,6'; antenna index 5 and 6 solutions
    antenna = '05,06'; antenna names '05' and '06 solutions
    
    timerange   Select time range to list
    default: ''-->all; examples:
    timerange='10:37:50.1'; list data for this sampling interval
    timerange='<10:37:25'; list data before 10:37:25
    
    correlation Select polarization correlations to list
    default: ''-->all; examples:
    correlation='RR LL'; list RR and LL correlations
    correlation='XX XY'; list XX and XY correlations
    
    scan        Select scans to list
    default: ''-->all; examples:
    scan='2'; list scan 2
    scan='>2'; list scan numbers greater than 2
    
    feed        (not yet implemented)
    
    array       (not yet implemented)
    
    observation Select by observation ID(s).
    default: ''-->all;
    example: observation='0' (select obsID 0)
    
    uvrange     Select baseline lengths to list.
    default: ''--> all; examples:
    uvrange='<5klambda'; less than 5 kilo-wavelengths
    Caution: Input units default to meters.
    Listed units are always wavelengths.
    
    average     (not yet implemented)
    
    showflags   (not yet implemented)
    
    pagerows    rows per page of listing
    default: 50; 0 --> do not paginate
    
    listfile    write output to disk; will not overwrite
    default: '' --> write to screen
    listfile = 'solutions.txt'
    
    


    """

    _info_group_ = """information, analysis"""
    _info_desc_ = """List measurement set visibilities."""

    def __call__( self, vis='', options='ap', datacolumn='data', field='', spw='*', selectdata=False, antenna='', timerange='', correlation='', scan='', feed='', array='', observation='', uvrange='', average='', showflags=False, pagerows=int(50), listfile='' ):
        schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'options': {'type': 'cStr', 'coerce': _coerce.to_str}, 'datacolumn': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'model', 'corrected', 'residual', 'float_data', 'data' ]}, 'field': {'type': 'cStr', 'coerce': _coerce.to_str}, 'spw': {'type': 'cStr', 'coerce': _coerce.to_str}, 'selectdata': {'type': 'cBool'}, 'antenna': {'type': 'cStr', 'coerce': _coerce.to_str}, 'timerange': {'type': 'cStr', 'coerce': _coerce.to_str}, 'correlation': {'type': 'cStr', 'coerce': _coerce.to_str}, 'scan': {'type': 'cStr', 'coerce': _coerce.to_str}, 'feed': {'type': 'cStr', 'coerce': _coerce.to_str}, 'array': {'type': 'cStr', 'coerce': _coerce.to_str}, 'observation': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cInt'}]}, 'uvrange': {'type': 'cStr', 'coerce': _coerce.to_str}, 'average': {'type': 'cStr', 'coerce': _coerce.to_str, 'allowed': [ 'none', 'chan', 'both', '', 'time' ]}, 'showflags': {'type': 'cBool'}, 'pagerows': {'type': 'cInt'}, 'listfile': {'type': 'cStr', 'coerce': _coerce.to_str}}
        doc = {'vis': vis, 'options': options, 'datacolumn': datacolumn, 'field': field, 'spw': spw, 'selectdata': selectdata, 'antenna': antenna, 'timerange': timerange, 'correlation': correlation, 'scan': scan, 'feed': feed, 'array': array, 'observation': observation, 'uvrange': uvrange, 'average': average, 'showflags': showflags, 'pagerows': pagerows, 'listfile': listfile}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'listvis', [ 'vis=' + repr(_pc.document['vis']), 'options=' + repr(_pc.document['options']), 'datacolumn=' + repr(_pc.document['datacolumn']), 'field=' + repr(_pc.document['field']), 'spw=' + repr(_pc.document['spw']), 'selectdata=' + repr(_pc.document['selectdata']), 'antenna=' + repr(_pc.document['antenna']), 'timerange=' + repr(_pc.document['timerange']), 'correlation=' + repr(_pc.document['correlation']), 'scan=' + repr(_pc.document['scan']), 'feed=' + repr(_pc.document['feed']), 'array=' + repr(_pc.document['array']), 'observation=' + repr(_pc.document['observation']), 'uvrange=' + repr(_pc.document['uvrange']), 'average=' + repr(_pc.document['average']), 'showflags=' + repr(_pc.document['showflags']), 'pagerows=' + repr(_pc.document['pagerows']), 'listfile=' + repr(_pc.document['listfile']) ] )
        task_result = None
        try:
            task_result = _listvis_t( _pc.document['vis'], _pc.document['options'], _pc.document['datacolumn'], _pc.document['field'], _pc.document['spw'], _pc.document['selectdata'], _pc.document['antenna'], _pc.document['timerange'], _pc.document['correlation'], _pc.document['scan'], _pc.document['feed'], _pc.document['array'], _pc.document['observation'], _pc.document['uvrange'], _pc.document['average'], _pc.document['showflags'], _pc.document['pagerows'], _pc.document['listfile'] )
        except Exception as exc:
            _except_log('listvis', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'listvis', task_result )
        return task_result

listvis = _listvis( )

