##################### generated by xml-casa (v2) from uvcontsub_old.xml #############
##################### f0f0b427965b14190ecdd5e50a5251e8 ##############################
from __future__ import absolute_import
import numpy
from casatools.typecheck import CasaValidator as _val_ctor
_pc = _val_ctor( )
from casatools.coercetype import coerce as _coerce
from casatools.errors import create_error_string
from .private.task_uvcontsub_old import uvcontsub_old as _uvcontsub_old_t
from casatasks.private.task_logging import start_log as _start_log
from casatasks.private.task_logging import end_log as _end_log
from casatasks.private.task_logging import except_log as _except_log

class _uvcontsub_old:
    """
    uvcontsub_old ---- Continuum fitting and subtraction in the uv plane

    --------- parameter descriptions ---------------------------------------------

    vis          Name of input MS.  Output goes to vis + ".contsub" (will be overwritten if already exists)
    field        Select field(s) using id(s) or name(s)
    fitspw       Spectral window:channel selection for fitting the continuum
    excludechans exclude Spectral window:channel selection in fitspw for fitting
    combine      Data axes to combine for the continuum estimation (none, or spw and/or scan)
    solint       Continuum fit timescale (int recommended!)
    fitorder     Polynomial order for the fits
    spw          Spectral window selection for output
    want_cont    Create vis + ".cont" to hold the continuum estimate.

    --------- examples -----------------------------------------------------------

    
    
    Continuum fitting and subtraction in the uv plane:
    
    This task estimates the continuum emission by fitting polynomials to
    the real and imaginary parts of the spectral windows and channels
    selected by fitspw.  This fit represents a model of the continuum in
    all channels.
    
    The fitted continuum spectrum is subtracted from all channels
    selected in spw, and the result (presumably only line emission)
    is stored in a new MS (vis + ".contsub"). If an MS
    with the output name already exists, it will be overwritten.
    It will read from the CORRECTED_DATA column of vis if it is present,
    or DATA if it is not.  Whichever column is read is presumed to have
    already been calibrated.
    
    If want_cont is True, the continuum fit is placed in a second new MS
    (vis + '.cont', also overwritten if it already exists).
    N.B. because the continuum model is necessarily a
    smoothed fit, images made with it are liable to have their field of
    view reduced in some strange way.  Images of the continuum should be
    made by simply excluding the line channels (and probably averaging the
    remaining ones) in clean.
    
    Keyword arguments:
    vis -- Name of input visibility file
    default: none; example: vis='ngc5921.ms'
    field -- Field selection for continuum estimation and subtraction.
    The estimation and subtraction is done for each selected field
    in turn.  (Run listobs to get lists of the ID and names.)
    default: field = '' means select all fields
    field = 1 # will get field_id=1 (if you give it an
    integer, it will retrieve the source with that index.
    field = '1328+307'  specifies source '1328+307'
    field = '13*' will retrieve '1328+307' and any other fields
    beginning with '13'
    fitspw -- Selection of spectral windows and channels to use in the
    fit for the continuum, using general spw:chan syntax.
    The ranges of channels also can be specified by frequencies as in
    the MS selection syntax (spw ids are required but '*' can be
    used, see the example below).
    See the note under combine.
    default: '' (all)
    example: fitspw='0:5~30;40~55'
    --> select the ranges by channels in the spw id 0
    fitspw='0:5~30;40~55,1:10~25;45~58,2'
    --> select channel ranges 5-30 and 40-55 for the spw id 0,
    10-25 and 45-58 for spwid 1, and use all channels for the spw id 2
    fitspw='0:113.767~114.528GHz;114.744~115.447GHz'
    --> select the ranges by frequencies in the spw id 0
    fitspw='0:113.767~114.528GHz;114.744~115.447GHz,1:111.892~112.654GHz;112.868~113.025GHz'
    --> select the different ranges by frequencies for the spw ids 0 and 1
    fitspw='*:113.767~114.528GHz;114.744~115.447GHz'
    --> select the same frequency ranges for all the relevant spws
    >>> expandable parameter for fitspw
    excludechans - if True, it will exclude the spws:channels specified in fitspw
    for the fit
    default: False (use fitspw for the fit)
    example: fitspw='0:114.528GHz~114.744GHz'; excludechans=True
    --> exclude the frequency range, 114.528GHz - 114.744GHz in the spw id 0
    combine -- Data axes to combine for the continuum estimate.
    It must include 'spw' if spw contains spws that are not in
    fitspw!
    default: '' --> solutions will break at scan, field, and spw
    boundaries according to solint
    Options: '', 'spw'', 'scan', or 'spw, scan'
    example: combine='spw' --> form spw-merged continuum estimate
    solint -- Timescale for per-baseline fit (units optional)
    default (recommended): 'int' --> no time averaging, do a
    fit for each integration and let the
    noisy fits average out in the image.
    
    example: solint='10s'  --> average to 10s before fitting
    10 or '10' --> '10s' (unitless: assumes seconds)
    options: 'int' --> per integration
    'inf' --> per scan
    
    If solint is longer than 'int', the continuum estimate can be
    corrupted by time smearing!
    
    fitorder -- Polynomial order for the fits of the continuum w.r.t.
    frequency.  fitorders > 1 are strongly discouraged
    because high order polynomials have more flexibility, may
    absorb line emission, and tend go wild at the edges of
    fitspw, which is not what you want.
    
    default: 0 (constant); example: fitorder=1
    
    spw -- Optional per spectral window selection of channels to include
    in the output.  See the note under combine.
    
    The spectral windows will be renumbered to start from 0, as in
    split.
    want_cont -- Create vis + '.cont' to hold the continuum estimate.
    default: 'False'; example: want_cont=True
    The continuum estimate will be placed in vis + '.cont'


    """

    _info_group_ = """manipulation"""
    _info_desc_ = """Continuum fitting and subtraction in the uv plane"""

    def __call__( self, vis='', field='', fitspw='', excludechans=False, combine='', solint='int', fitorder=int(0), spw='', want_cont=False ):
        schema = {'vis': {'type': 'cReqPath', 'coerce': _coerce.expand_path}, 'field': {'anyof': [{'type': 'cStr', 'coerce': _coerce.to_str}, {'type': 'cStrVec', 'coerce': [_coerce.to_list,_coerce.to_strvec]}, {'type': 'cInt'}, {'type': 'cIntVec', 'coerce': [_coerce.to_list,_coerce.to_intvec]}]}, 'fitspw': {'type': 'cStr', 'coerce': _coerce.to_str}, 'excludechans': {'type': 'cBool'}, 'combine': {'type': 'cStr', 'coerce': _coerce.to_str}, 'solint': {'type': 'cVariant', 'coerce': [_coerce.to_variant]}, 'fitorder': {'type': 'cInt'}, 'spw': {'type': 'cStr', 'coerce': _coerce.to_str}, 'want_cont': {'type': 'cBool'}}
        doc = {'vis': vis, 'field': field, 'fitspw': fitspw, 'excludechans': excludechans, 'combine': combine, 'solint': solint, 'fitorder': fitorder, 'spw': spw, 'want_cont': want_cont}
        assert _pc.validate(doc,schema), create_error_string(_pc.errors)
        _logging_state_ = _start_log( 'uvcontsub_old', [ 'vis=' + repr(_pc.document['vis']), 'field=' + repr(_pc.document['field']), 'fitspw=' + repr(_pc.document['fitspw']), 'excludechans=' + repr(_pc.document['excludechans']), 'combine=' + repr(_pc.document['combine']), 'solint=' + repr(_pc.document['solint']), 'fitorder=' + repr(_pc.document['fitorder']), 'spw=' + repr(_pc.document['spw']), 'want_cont=' + repr(_pc.document['want_cont']) ] )
        task_result = None
        try:
            task_result = _uvcontsub_old_t( _pc.document['vis'], _pc.document['field'], _pc.document['fitspw'], _pc.document['excludechans'], _pc.document['combine'], _pc.document['solint'], _pc.document['fitorder'], _pc.document['spw'], _pc.document['want_cont'] )
        except Exception as exc:
            _except_log('uvcontsub_old', exc)
            raise
        finally:
            task_result = _end_log( _logging_state_, 'uvcontsub_old', task_result )
        return task_result

uvcontsub_old = _uvcontsub_old( )

